#include "common.h"
#include "portinfo.h"
#include "token.h"
#include "parse.h"
#include "log.h"
#include "util.h"

#define PORTNAMELEN 32
#define SADDRBUF 64
/* ----------------------------------------------------------------- */
const PORTOPTION*
in_word_set_portoption (register const char *str, register int len);
/* ----------------------------------------------------------------- */

void Portinfo::init()
{
  *hostname = 0;
  sa.sin_addr.s_addr = 0;
  sa.sin_port = 0;
  flags = 0;
}

void PortinfoList::debug()
{
  Portinfo *p = top;
  while (p) {
    PUTERR(0, ("\t%s(%d)%s\n", p->hostname, ntohs(p->sa.sin_port),
	       p->flags & PO_SSL ? " with SSL" : ""));
    if (p->sa_from_if.sin_addr.s_addr) {
      char buf[SADDRBUF];
      PUTERR(0, ("\tbind to `%s'",
		 addr2str(p->sa_from_if.sin_addr.s_addr, buf)));
    }
    p = p->next;
  }
}

/* ---------------------------------------------------------------- *
   =NAME=
      Portinfo::set - $B%[%9%HL>$H%]!<%HL>$rF@$k(B
   =SYNOPSYS=
      int Portinfo::set(char *pp)

      pp   - $BJ8;zNs(B, $B4X?t<B9T8e!"(Bpp $B$O(B <port>$B$N<!$NJ8;z$r;X$9(B
   =RETURN=
      0 - $B0[>o=*N;(B
      1 - $B@5>o=*N;(B
   =DESCRIPTION=
      $B<!$N9=J8$NJ8;zNs$+$i%[%9%HL>$H%]!<%HL>$rF@$k(B

      [ <host> : ] <port> [ with <option> ]
      ($B3F%H!<%/%s$N4V$K#1$D0J>e$N6uGr$NA^F~$O5v$5$l$k(B)

      <host> := $B%[%9%HL>!#:GBg(B (MAXHOSTNAMELEN - 1) $BJ8;z$^$G3JG<$5$l$k!#(B
                $B>JN,$5$l$k$H!"(B host $B$K(B "localhost" $B$,3JG<$5$l$k!#(B
      <port> := $B%]!<%HL>!#(B
      <option> := $B%*%W%7%g%sJ8;zNs(B, $B%+%s%^$GJ#?t;XDj$G$-$k(B
                  SSL -- SSL$B$r;H$&(B
 * ----------------------------------------------------------------- */
int Portinfo::set(char *p)
{
  char c;
  char sport[PORTNAMELEN];

  /* pi $B$N=i4|2=(B */
  init();

  /* $B:G=i$N6uGr$r%9%-%C%W(B */
  skipSpace(&p);

  /* <host> $B$N<h$j=P$7(B */
  {
    char *hostp = hostname;
    int len;

    len = MAXHOSTNAMELEN;
    while (c = *p, c && --len && c != ':' && c != ' ' && c != '\t') {
      *hostp++ = c; p++;
    }
    *hostp = 0;
    while (c = *p, c && c != ':' && c != ' ' && c != '\t') p++;
    skipSpace(&p);

    /* $B$3$N%V%m%C%/$r=hM}$9$k$H(B... */
    /* c <-- $B:G=*=hM}J8;z$,3JG<$5$l$k(B */
    /* hostname <-- $B%[%9%HL>(B($B%]!<%HL>$+$b$7$l$J$$(B) $B$,3JG<$5$l$k(B */
  }

  /* ':' $B$,$J$1$l$P!"(Bhost$B$K3JG<$5$l$?J8;zNs$O(B port $BHV9f$G$"$k(B */
  /*                 host$BL>$O(B 'localhost' $B$G$"$k(B */
  if (c != ':') {
    STRCPY(sport, hostname);
    STRCPY(hostname, "localhost");
  } else {
    /* $B%]!<%HHV9f$rFI$_$H$k(B */
    char *portp = sport;
    int len;
    
    p++;
    skipSpace(&p);
    len = PORTNAMELEN;
    while (c = *p, c && --len && c != ' ' && c != '\t') {
      *portp++ = c; p++;
    }
    *portp = 0;
    skipUntilSpace(&p); /* $BD92a$.$k%]!<%HL>$r$9$CHt$P$9(B */
  }
  skipSpace(&p);
  if (!STRNLOWCMP(p, "with", 4)) {
    /* with $B0J2<$N%*%W%7%g%s$rFI$_$H$k(B */
    skipUntilSpace(&p);
    while (skipSpace(&p), *p) {
      char buf[32];
      char *bufp = buf;
      int len = 0;
      const PORTOPTION *pop;
      int rlen = 32;

      while (c = *p, c && c != ',' && c != ' '&& c != '\t' && --rlen) {
	*bufp++ = *p++;
	len++;
      }
      *bufp = 0;
      pop = in_word_set_portoption(buf, len);
      if (!pop) {
	fprintf(stderr, "Unknown port option `%s'\n", buf);
	return FALSE;
      }
      flags |= pop->value;
      if (c == ',') p++;
    }
  }
  /* skipUntilSpace(&p);*/
  if (!getPort(sport, &sa.sin_port))
    return FALSE;
  if (!getHost(hostname, (DWORD*)&sa.sin_addr.s_addr))
    return FALSE;

  skipSpace(&p);
  if (!STRNLOWCMP(p, "from", 4)) {
    /* from $B0J2<$N%*%W%7%g%s$rFI$_$H$k(B */
    skipUntilSpace(&p);
    skipSpace(&p);
    char from_hostname[MAXHOSTNAMELEN];
    char *hostp = from_hostname;
    int len;

    len = MAXHOSTNAMELEN;
    while (c = *p, c && --len && c != ':' && c != ' ' && c != '\t') {
      *hostp++ = c; p++;
    }
    *hostp = 0;
    while (c = *p, c && c != ':' && c != ' ' && c != '\t') p++;
    skipSpace(&p);
    if (!*from_hostname) return FALSE;
    if (!getHost(from_hostname, (DWORD*)&sa_from_if.sin_addr.s_addr))
      return FALSE;
  }
  
  return TRUE;
}
