#include "config.h"

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#include <unistd.h> /* select() */

#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */

#include "Config.h"
#include "servicetype.h"

/* ----------------------------------------------------------------- *
   =FUNC=
       dump_telnet - dump telnet session data
 * ----------------------------------------------------------------- */
#ifndef NO_DEBUG
void dump_telnet(char *buf, int len)
{
  char sbuf[256], *p = sbuf;
  while (len--) {
    unsigned char c = *buf++;
    if (c == 0xff) {
      strcpy(p, "IAC "); p+= 4; continue;
    }
    sprintf(p, "%02x ", c); p+= 3;
  }
  PUTERR(2, ("Receive Data: %s\n", sbuf));
}
#endif

/* ----------------------------------------------------------------- *
   =FUNC=
       st_SSLTelnet_init - Initialize SSL Telnet session
   =RETURN=
       0 - success
 * ----------------------------------------------------------------- */
int st_SSLTelnet_init(Config *cfg)
{
  cfg->accept_port.getPortinfo()->resetSSL();
  return 0;
}

/* ----------------------------------------------------------------- *
   =FUNC=
       st_SSLTelnet_func - do SSL Telnet session
   =RETURN=
       0 - success
       1 - failed(continue to relay)
       2 - critical error
/* ----------------------------------------------------------------- */
int st_SSLTelnet_common_func(Config *cfg, int csoc, int mode)
{
  int session = 0;
  char buf[64];
  int ret;
  int rlen, rflag = 0;

#ifndef NO_DEBUG
  PUTERR(2, ("SSLTelnet session begin\n"));
#endif

  while (1) {
    switch (session) {
    case 0: /* send to "IAC DO AUTHENTICATION " */
#ifdef DEBUG_SSLTELNET
      PUTERR(2, ("SSLTelnet: send \"IAC DO AUTHENTICATION\"\n"));
#endif /* DEBUG_SSLTELNET */
      ret = send(csoc, "\xff\xfd\x25", 3, 0);
      rlen = 3;
      break;

    case 1: /* get reply */
      if (!strncmp(buf, "\xff\xfb\x25", 3)) {
	/* get "IAC WILL AUTHENTICATION" */
#ifdef DEBUG_SSLTELNET
	PUTERR(2, ("SSLTelnet: get \"IAC WILL AUTHENTICATION\"\n"));
	PUTERR(2, ("SSLTelnet: send \"IAC SB AUTHENTICATION SEND 07 00\"\n"));
#endif /* DEBUG_SSLTELNET */
	send(csoc, "\xff\xfa\x25\x01\x07\x00\xff\xf0", 6, 0);
	ret = send(csoc, "\xff\xf0", 2, 0);
	rlen = 255;
	rflag = 1;
	break;
      } else if (!strncmp(buf, "\xff\xfc\x25", 3)) {
	/* get "IAC WONT AUTHENTICATION" */
#ifdef DEBUG_SSLTELNET
	PUTERR(2, ("SSLTelnet: get \"IAC WONT AUTHENTICATION\"\n"));
#endif /* DEBUG_SSLTELNET */
	if (!mode) {
	  PUTERR(0, ("SSLTelnet: client doesn't the SSL telnet\n"));
	  return 2;
	} else {
	  PUTERR(1, ("SSLTelnet: warning: client doesn't the SSL telnet\n"));
	  return 1;
	}
      }
      if (!mode) {
	PUTERR(0, ("SSLTelnet: client doesn't the SSL telnet\n"));
	return 2;
      } else {
	PUTERR(1, ("SSLTelnet: warning: bad response from client\n"));
	return 1;
      }

    case 2: /* get reply */
      if (!strncmp(buf, "\xff\xfa\x25\x00\x07\x00\x01", 7)) {
	/* get "IAC SB AUTHENTICATION IS 07 00 01" */
#ifdef DEBUG_SSLTELNET
	PUTERR(2, ("SSLTelnet: get \"IAC SB AUTHENTICATION IS 07 00 01\"\n"));
	PUTERR(2, ("SSLTelnet: send \"IAC SB AUTHENTICATION REPLY 07 00 02\"\n"));
#endif /* DEBUG_SSLTELNET */
	send(csoc, "\xff\xfa\x25\x02\x07\x00\x02", 7, 0);
	ret = send(csoc, "\xff\xf0", 2, 0);
	cfg->accept_port.getPortinfo()->setSSL();
	return 0;
      }
      if (!mode) {
	PUTERR(0, ("SSLTelnet: client doesn't the SSL telnet\n"));
#ifdef DEBUG_SSLTELNET
	dump_telnet(buf, rlen);
#endif
	return 2;
      } else {
	PUTERR(1, ("SSLTelnet: bad response from client\n"));
	return 1;
      }
    }
    if (ret < 0) {
      PUTERR(0, ("SSLTelnet: Fail to send(): %s\n", strerror(errno)));
      return 2;
    }
    session++;

#if 1
    {
      fd_set rmask;
      fd_set fds;
      int n;
      int max = 64;

      while (1) {
        FD_ZERO(&fds);
	FD_SET(csoc, &fds);
	
	memcpy(&rmask, &fds, sizeof(fd_set));
	n = ::select(max, &rmask, 0, 0, 0);
	if (n) {
	  break;
	}
      }
    }
#endif
    {
      int len = 0;
      char *p = buf;

      while (1) {
	ret = recv(csoc, buf, rlen, 0);
	if (ret < 0) {
	  PUTERR(0, ("SSLTelnet: Fail to read(): %s\n",
		     strerror(errno)));
	  return 1;
	}
	p+= ret;
	len += ret;
#ifdef DEBUG_SSLTELNET
	PUTERR(0, ("%d/%d received\n", len, rlen));
#endif
	if (len >= rlen || rflag) break;
      }
      rflag = 0;
    }
  }
  return 0;
}

int st_SSLTelnet_func(Config *cfg, int csoc)
{
  return st_SSLTelnet_common_func(cfg, csoc, 0);
}

int st_SSLTelnet_or_telnet_func(Config *cfg, int csoc)
{
  return st_SSLTelnet_common_func(cfg, csoc, 1);
}


/* ================================================================= */
serviceTypeFunc_t serviceTypeFunc[] = {
  "NONE", 0, 0, 
  "SSL Telnet", st_SSLTelnet_init, st_SSLTelnet_func,
  "SSL Telnet or Telnet", st_SSLTelnet_init, st_SSLTelnet_or_telnet_func
  };
